@@ -24,15 +24,17 @@ Command-line interface to the OpenStack Magnum API.
 
 from __future__ import print_function
 import argparse
-import getpass
 import logging
 import os
 import sys
 
 from oslo_utils import encodeutils
+from oslo_utils import importutils
 from oslo_utils import strutils
 import six
 
+profiler = importutils.try_import("osprofiler.profiler")
+
 HAS_KEYRING = False
 all_errors = ValueError
 try:
@@ -51,6 +53,7 @@ except ImportError:
 
 from magnumclient.common import cliutils
 from magnumclient import exceptions as exc
+from magnumclient.i18n import _
 from magnumclient.v1 import client as client_v1
 from magnumclient.v1 import shell as shell_v1
 from magnumclient import version
@@ -76,126 +79,6 @@ def positive_non_zero_float(text):
     return value
 
 
-class SecretsHelper(object):
-    def __init__(self, args, client):
-        self.args = args
-        self.client = client
-        self.key = None
-
-    def _validate_string(self, text):
-        if text is None or len(text) == 0:
-            return False
-        return True
-
-    def _make_key(self):
-        if self.key is not None:
-            return self.key
-        keys = [
-            self.client.auth_url,
-            self.client.projectid,
-            self.client.user,
-            self.client.region_name,
-            self.client.endpoint_type,
-            self.client.service_type,
-            self.client.service_name,
-            self.client.volume_service_name,
-        ]
-        for (index, key) in enumerate(keys):
-            if key is None:
-                keys[index] = '?'
-            else:
-                keys[index] = str(keys[index])
-        self.key = "/".join(keys)
-        return self.key
-
-    def _prompt_password(self, verify=True):
-        pw = None
-        if hasattr(sys.stdin, 'isatty') and sys.stdin.isatty():
-            # Check for Ctl-D
-            try:
-                while True:
-                    pw1 = getpass.getpass('OS Password: ')
-                    if verify:
-                        pw2 = getpass.getpass('Please verify: ')
-                    else:
-                        pw2 = pw1
-                    if pw1 == pw2 and self._validate_string(pw1):
-                        pw = pw1
-                        break
-            except EOFError:
-                pass
-        return pw
-
-    def save(self, auth_token, management_url, tenant_id):
-        if not HAS_KEYRING or not self.args.os_cache:
-            return
-        if (auth_token == self.auth_token and
-           management_url == self.management_url):
-            # Nothing changed....
-            return
-        if not all([management_url, auth_token, tenant_id]):
-            raise ValueError("Unable to save empty management url/auth token")
-        value = "|".join([str(auth_token),
-                          str(management_url),
-                          str(tenant_id)])
-        keyring.set_password("magnumclient_auth", self._make_key(), value)
-
-    @property
-    def password(self):
-        if self._validate_string(self.args.os_password):
-            return self.args.os_password
-        verify_pass = (
-            strutils.bool_from_string(cliutils.env("OS_VERIFY_PASSWORD"))
-        )
-        return self._prompt_password(verify_pass)
-
-    @property
-    def management_url(self):
-        if not HAS_KEYRING or not self.args.os_cache:
-            return None
-        management_url = None
-        try:
-            block = keyring.get_password('magnumclient_auth',
-                                         self._make_key())
-            if block:
-                _token, management_url, _tenant_id = block.split('|', 2)
-        except all_errors:
-            pass
-        return management_url
-
-    @property
-    def auth_token(self):
-        # Now is where it gets complicated since we
-        # want to look into the keyring module, if it
-        # exists and see if anything was provided in that
-        # file that we can use.
-        if not HAS_KEYRING or not self.args.os_cache:
-            return None
-        token = None
-        try:
-            block = keyring.get_password('magnumclient_auth',
-                                         self._make_key())
-            if block:
-                token, _management_url, _tenant_id = block.split('|', 2)
-        except all_errors:
-            pass
-        return token
-
-    @property
-    def tenant_id(self):
-        if not HAS_KEYRING or not self.args.os_cache:
-            return None
-        tenant_id = None
-        try:
-            block = keyring.get_password('magnumclient_auth',
-                                         self._make_key())
-            if block:
-                _token, _management_url, tenant_id = block.split('|', 2)
-        except all_errors:
-            pass
-        return tenant_id
-
-
 class MagnumClientArgumentParser(argparse.ArgumentParser):
 
     def __init__(self, *args, **kwargs):
@@ -242,19 +125,20 @@ class OpenStackMagnumShell(object):
         parser.add_argument('--debug',
                             default=False,
                             action='store_true',
-                            help="Print debugging output.")
+                            help=_("Print debugging output."))
 
         parser.add_argument('--os-cache',
                             default=strutils.bool_from_string(
                                 cliutils.env('OS_CACHE', default=False)),
                             action='store_true',
-                            help="Use the auth token cache. Defaults to False "
-                            "if env[OS_CACHE] is not set.")
+                            help=_("Use the auth token cache. Defaults to "
+                                   "False if env[OS_CACHE] is not set."))
 
         parser.add_argument('--os-region-name',
                             metavar='<region-name>',
                             default=os.environ.get('OS_REGION_NAME'),
-                            help='Region name. Default=env[OS_REGION_NAME].')
+                            help=_('Region name. '
+                                   'Default=env[OS_REGION_NAME].'))
 
 
 # TODO(mattf) - add get_timings support to Client
@@ -273,41 +157,41 @@ class OpenStackMagnumShell(object):
         parser.add_argument('--os-auth-url',
                             metavar='<auth-auth-url>',
                             default=cliutils.env('OS_AUTH_URL', default=None),
-                            help='Defaults to env[OS_AUTH_URL].')
+                            help=_('Defaults to env[OS_AUTH_URL].'))
 
         parser.add_argument('--os-user-id',
                             metavar='<auth-user-id>',
                             default=cliutils.env('OS_USER_ID', default=None),
-                            help='Defaults to env[OS_USER_ID].')
+                            help=_('Defaults to env[OS_USER_ID].'))
 
         parser.add_argument('--os-username',
                             metavar='<auth-username>',
                             default=cliutils.env('OS_USERNAME', default=None),
-                            help='Defaults to env[OS_USERNAME].')
+                            help=_('Defaults to env[OS_USERNAME].'))
 
         parser.add_argument('--os-user-domain-id',
                             metavar='<auth-user-domain-id>',
                             default=cliutils.env('OS_USER_DOMAIN_ID',
                                                  default=None),
-                            help='Defaults to env[OS_USER_DOMAIN_ID].')
+                            help=_('Defaults to env[OS_USER_DOMAIN_ID].'))
 
         parser.add_argument('--os-user-domain-name',
                             metavar='<auth-user-domain-name>',
                             default=cliutils.env('OS_USER_DOMAIN_NAME',
                                                  default=None),
-                            help='Defaults to env[OS_USER_DOMAIN_NAME].')
+                            help=_('Defaults to env[OS_USER_DOMAIN_NAME].'))
 
         parser.add_argument('--os-project-id',
                             metavar='<auth-project-id>',
                             default=cliutils.env('OS_PROJECT_ID',
                                                  default=None),
-                            help='Defaults to env[OS_PROJECT_ID].')
+                            help=_('Defaults to env[OS_PROJECT_ID].'))
 
         parser.add_argument('--os-project-name',
                             metavar='<auth-project-name>',
                             default=cliutils.env('OS_PROJECT_NAME',
                                                  default=None),
-                            help='Defaults to env[OS_PROJECT_NAME].')
+                            help=_('Defaults to env[OS_PROJECT_NAME].'))
 
         parser.add_argument('--os-tenant-id',
                             metavar='<auth-tenant-id>',
@@ -325,29 +209,30 @@ class OpenStackMagnumShell(object):
                             metavar='<auth-project-domain-id>',
                             default=cliutils.env('OS_PROJECT_DOMAIN_ID',
                                                  default=None),
-                            help='Defaults to env[OS_PROJECT_DOMAIN_ID].')
+                            help=_('Defaults to env[OS_PROJECT_DOMAIN_ID].'))
 
         parser.add_argument('--os-project-domain-name',
                             metavar='<auth-project-domain-name>',
                             default=cliutils.env('OS_PROJECT_DOMAIN_NAME',
                                                  default=None),
-                            help='Defaults to env[OS_PROJECT_DOMAIN_NAME].')
+                            help=_('Defaults to '
+                                   'env[OS_PROJECT_DOMAIN_NAME].'))
 
         parser.add_argument('--os-token',
                             metavar='<auth-token>',
                             default=cliutils.env('OS_TOKEN', default=None),
-                            help='Defaults to env[OS_TOKEN].')
+                            help=_('Defaults to env[OS_TOKEN].'))
 
         parser.add_argument('--os-password',
                             metavar='<auth-password>',
                             default=cliutils.env('OS_PASSWORD',
                                                  default=None),
-                            help='Defaults to env[OS_PASSWORD].')
+                            help=_('Defaults to env[OS_PASSWORD].'))
 
         parser.add_argument('--service-type',
                             metavar='<service-type>',
-                            help='Defaults to container-infra for all '
-                                 'actions.')
+                            help=_('Defaults to container-infra for all '
+                                   'actions.'))
         parser.add_argument('--service_type',
                             help=argparse.SUPPRESS)
 
@@ -361,7 +246,7 @@ class OpenStackMagnumShell(object):
                             metavar='<os-endpoint-type>',
                             default=cliutils.env('OS_ENDPOINT_TYPE',
                                                  default=None),
-                            help='Defaults to env[OS_ENDPOINT_TYPE]')
+                            help=_('Defaults to env[OS_ENDPOINT_TYPE]'))
 
         parser.add_argument('--os-interface',
                             metavar='<os-interface>',
@@ -373,7 +258,7 @@ class OpenStackMagnumShell(object):
         parser.add_argument('--os-cloud',
                             metavar='<auth-cloud>',
                             default=cliutils.env('OS_CLOUD', default=None),
-                            help='Defaults to env[OS_CLOUD].')
+                            help=_('Defaults to env[OS_CLOUD].'))
 
         # NOTE(dtroyer): We can't add --endpoint_type here due to argparse
         #                thinking usage-list --end is ambiguous; but it
@@ -387,24 +272,24 @@ class OpenStackMagnumShell(object):
                             default=cliutils.env(
                                 'MAGNUM_API_VERSION',
                                 default='latest'),
-                            help='Accepts "api", '
-                                 'defaults to env[MAGNUM_API_VERSION].')
+                            help=_('Accepts "api", '
+                                   'defaults to env[MAGNUM_API_VERSION].'))
         parser.add_argument('--magnum_api_version',
                             help=argparse.SUPPRESS)
 
         parser.add_argument('--os-cacert',
                             metavar='<ca-certificate>',
                             default=cliutils.env('OS_CACERT', default=None),
-                            help='Specify a CA bundle file to use in '
-                            'verifying a TLS (https) server certificate. '
-                            'Defaults to env[OS_CACERT].')
+                            help=_('Specify a CA bundle file to use in '
+                                   'verifying a TLS (https) server '
+                                   'certificate. Defaults to env[OS_CACERT].'))
 
         parser.add_argument('--os-endpoint-override',
                             metavar='<endpoint-override>',
                             default=cliutils.env('OS_ENDPOINT_OVERRIDE',
                                                  default=None),
-                            help="Use this API endpoint instead of the "
-                            "Service Catalog.")
+                            help=_("Use this API endpoint instead of the "
+                                   "Service Catalog."))
         parser.add_argument('--bypass-url',
                             metavar='<bypass-url>',
                             default=cliutils.env('BYPASS_URL', default=None),
@@ -417,7 +302,23 @@ class OpenStackMagnumShell(object):
                             default=cliutils.env('MAGNUMCLIENT_INSECURE',
                                                  default=False),
                             action='store_true',
-                            help="Do not verify https connections")
+                            help=_("Do not verify https connections"))
+
+        if profiler:
+            parser.add_argument('--profile',
+                                metavar='HMAC_KEY',
+                                default=cliutils.env('OS_PROFILE',
+                                                     default=None),
+                                help='HMAC key to use for encrypting context '
+                                'data for performance profiling of operation. '
+                                'This key should be the value of the HMAC key '
+                                'configured for the OSprofiler middleware in '
+                                'magnum; it is specified in the Magnum '
+                                'configuration file at '
+                                '"/etc/magnum/magnum.conf". '
+                                'Without the key, profiling will not be '
+                                'triggered even if OSprofiler is enabled on '
+                                'the server side.')
 
         return parser
 
@@ -459,6 +360,7 @@ class OpenStackMagnumShell(object):
             desc = callback.__doc__ or ''
             action_help = desc.strip()
             arguments = getattr(callback, 'arguments', [])
+            group_args = getattr(callback, 'deprecated_groups', [])
 
             subparser = (
                 subparsers.add_parser(command,
@@ -471,6 +373,12 @@ class OpenStackMagnumShell(object):
                                    action='help',
                                    help=argparse.SUPPRESS,)
             self.subcommands[command] = subparser
+
+            for (old_info, new_info, req) in group_args:
+                group = subparser.add_mutually_exclusive_group(required=req)
+                group.add_argument(*old_info[0], **old_info[1])
+                group.add_argument(*new_info[0], **new_info[1])
+
             for (args, kwargs) in arguments:
                 subparser.add_argument(*args, **kwargs)
             subparser.set_defaults(func=callback)
@@ -515,6 +423,36 @@ class OpenStackMagnumShell(object):
             api_major_version = versions[0]
             return (api_major_version, magnum_api_version)
 
+    def _ensure_auth_info(self, args):
+        if not cliutils.isunauthenticated(args.func):
+            if (not (args.os_token and
+                     (args.os_auth_url or args.os_endpoint_override)) and
+                not args.os_cloud
+                ):
+
+                if not (args.os_username or args.os_user_id):
+                    raise exc.CommandError(
+                        "You must provide a username via either --os-username "
+                        "or via env[OS_USERNAME]"
+                    )
+                if not args.os_password:
+                    raise exc.CommandError(
+                        "You must provide a password via either "
+                        "--os-password, env[OS_PASSWORD], or prompted "
+                        "response"
+                    )
+                if (not args.os_project_name and not args.os_project_id):
+                    raise exc.CommandError(
+                        "You must provide a project name or project id via "
+                        "--os-project-name, --os-project-id, "
+                        "env[OS_PROJECT_NAME] or env[OS_PROJECT_ID]"
+                    )
+                if not args.os_auth_url:
+                    raise exc.CommandError(
+                        "You must provide an auth url via either "
+                        "--os-auth-url or via env[OS_AUTH_URL]"
+                    )
+
     def main(self, argv):
 
         # NOTE(Christoph Jansen): With Python 3.4 argv somehow becomes a Map.
@@ -567,34 +505,8 @@ class OpenStackMagnumShell(object):
         args.os_project_id = (args.os_project_id or args.os_tenant_id)
         args.os_project_name = (args.os_project_name or args.os_tenant_name)
 
-        if not cliutils.isunauthenticated(args.func):
-            if (not (args.os_token and
-                     (args.os_auth_url or args.os_endpoint_override)) and
-                not args.os_cloud
-                ):
+        self._ensure_auth_info(args)
 
-                if not (args.os_username or args.os_user_id):
-                    raise exc.CommandError(
-                        "You must provide a username via either --os-username "
-                        "or via env[OS_USERNAME]"
-                    )
-                if not args.os_password:
-                    raise exc.CommandError(
-                        "You must provide a password via either "
-                        "--os-password, env[OS_PASSWORD], or prompted "
-                        "response"
-                    )
-                if (not args.os_project_name and not args.os_project_id):
-                    raise exc.CommandError(
-                        "You must provide a project name or project id via "
-                        "--os-project-name, --os-project-id, "
-                        "env[OS_PROJECT_NAME] or env[OS_PROJECT_ID]"
-                    )
-                if not args.os_auth_url:
-                    raise exc.CommandError(
-                        "You must provide an auth url via either "
-                        "--os-auth-url or via env[OS_AUTH_URL]"
-                    )
         try:
             client = {
                 '1': client_v1,
@@ -609,6 +521,10 @@ class OpenStackMagnumShell(object):
         if args.os_interface.endswith('URL'):
             args.os_interface = args.os_interface[:-3]
 
+        kwargs = {}
+        if profiler:
+            kwargs["profile"] = args.profile
+
         self.cs = client.Client(
             cloud=args.os_cloud,
             user_id=args.os_user_id,
@@ -628,9 +544,42 @@ class OpenStackMagnumShell(object):
             interface=args.os_interface,
             insecure=args.insecure,
             api_version=args.magnum_api_version,
+            **kwargs
         )
 
-        args.func(self.cs, args)
+        self._check_deprecation(args.func, argv)
+        try:
+            args.func(self.cs, args)
+        except (cliutils.DuplicateArgs, cliutils.MissingArgs):
+            self.do_help(args)
+            raise
+
+        if profiler and args.profile:
+            trace_id = profiler.get().get_base_id()
+            print("To display trace use the command:\n\n"
+                  "  osprofiler trace show --html %s " % trace_id)
+
+    def _check_deprecation(self, func, argv):
+        if not hasattr(func, 'deprecated_groups'):
+            return
+
+        for (old_info, new_info, required) in func.deprecated_groups:
+            old_param = old_info[0][0]
+            new_param = new_info[0][0]
+            old_value, new_value = None, None
+            for i in range(len(argv)):
+                cur_arg = argv[i]
+                if cur_arg == old_param:
+                    old_value = argv[i + 1]
+                elif cur_arg == new_param[0]:
+                    new_value = argv[i + 1]
+
+            if old_value and not new_value:
+                print(
+                    'WARNING: The %s parameter is deprecated and will be '
+                    'removed in a future release. Use the %s parameter to '
+                    'avoid seeing this message.'
+                    % (old_param, new_param))
 
     def _dump_timings(self, timings):
         class Tyme(object):
@@ -662,7 +611,7 @@ class OpenStackMagnumShell(object):
         print(' '.join(commands | options))
 
     @cliutils.arg('command', metavar='<subcommand>', nargs='?',
-                  help='Display help for <subcommand>.')
+                  help=_('Display help for <subcommand>.'))
     def do_help(self, args):
         """Display help about this program or one of its subcommands."""
         # NOTE(jamespage): args.command is not guaranteed with python >= 3.4
