@@ -18,6 +18,11 @@ from magnumclient.v1 import baseunit
 CREATION_ATTRIBUTES = baseunit.CREATION_ATTRIBUTES
 CREATION_ATTRIBUTES.append('cluster_template_id')
 CREATION_ATTRIBUTES.append('create_timeout')
+CREATION_ATTRIBUTES.append('keypair')
+CREATION_ATTRIBUTES.append('docker_volume_size')
+CREATION_ATTRIBUTES.append('labels')
+CREATION_ATTRIBUTES.append('master_flavor_id')
+CREATION_ATTRIBUTES.append('flavor_id')
 
 
 class Cluster(baseunit.BaseTemplate):
